import { defineStore } from "pinia"
import { reactive, ref, watch } from "vue"
import { checkLoggedIn, getAtk } from "@/stores/userinfo"
import { buildRequestUrl, request } from "@/scripts/request"
import { useRoute } from "vue-router"

export const useChannels = defineStore("channels", () => {
  let socket: WebSocket
  let reconnectCount = 0

  const done = ref(false)

  const show = reactive({
    members: false,
    editor: false,
    delete: false,

    messages: {
      delete: false
    }
  })

  const related = reactive<{ [id: string]: any }>({
    edit_to: null,
    manage_to: null,
    delete_to: null,

    messages: {
      edit_to: null,
      reply_to: null,
      delete_to: null
    }
  })

  const available = ref<any[]>([])
  const current = ref<any>(null)
  const messages = ref<any[]>([])

  const route = useRoute()
  watch(
    () => route.params.channel,
    (val) => {
      if (!val) {
        messages.value = []
        current.value = null
      }
    },
    { immediate: true }
  )

  async function list() {
    if (!(await checkLoggedIn())) return

    const res = await request("messaging", "/api/channels/me/available", {
      headers: { Authorization: `Bearer ${await getAtk()}` }
    })
    if (res.status !== 200) {
      throw new Error(await res.text())
    } else {
      available.value = await res.json()
    }
  }

  async function connect() {
    if (!(await checkLoggedIn())) return

    const uri = buildRequestUrl("messaging", "/api/unified").replace("http", "ws")

    socket = new WebSocket(uri + `?tk=${await getAtk() as string}`)

    socket.addEventListener("open", (event) => {
      console.log("[MESSAGING] The unified websocket has been established... ", event.type)
      reconnectCount = 0
    })
    socket.addEventListener("close", (event) => {
      console.warn("[MESSAGING] The unified websocket is disconnected... ", event.reason, event.code)
      if(reconnectCount <= 3) {
        connect().then(() => {
          console.warn("[MESSAGING] Now reconnecting!")
          reconnectCount++
        })
      }
    })
    socket.addEventListener("message", (event) => {
      const data = JSON.parse(event.data)
      const payload = data["p"]
      if (payload?.channel_id === current.value.id) {
        switch (data["w"]) {
          case "messages.new":
            messages.value.unshift(payload)
            break
          case "messages.update":
            messages.value = messages.value.map((x) => {
              if (x.id === payload.id) return payload
              else return x
            })
            break
          case "messages.burnt":
            messages.value = messages.value.filter((x) => {
              return x.id !== payload.id
            })
            break
        }
      }
    })
  }

  function disconnect() {
    socket.close()
  }

  return { done, show, related, available, current, messages, list, connect, disconnect }
})